@@ -1,5 +1,5 @@
 /*
- * "$Id: ipp.c 5724 2006-07-12 19:42:35Z mike $"
+ * "$Id: ipp.c 6061 2006-10-23 00:26:52Z mike $"
  *
  *   IPP backend for the Common UNIX Printing System (CUPS).
  *
@@ -107,6 +107,7 @@ main(int  argc,				/* I - Number of command-line args */
 		hostname[1024],		/* Hostname */
 		username[255],		/* Username info */
 		resource[1024],		/* Resource info (printer name) */
+		addrname[256],		/* Address name */
 		*optptr,		/* Pointer to URI options */
 		name[255],		/* Name of option */
 		value[255],		/* Value of option */
@@ -132,13 +133,13 @@ main(int  argc,				/* I - Number of command-line args */
   ipp_attribute_t *format_sup;		/* document-format-supported */
   ipp_attribute_t *printer_state;	/* printer-state attribute */
   ipp_attribute_t *printer_accepting;	/* printer-is-accepting-jobs */
-  int		copies;			/* Number of copies remaining */
+  int		copies,			/* Number of copies for job */
+		copies_remaining;	/* Number of copies remaining */
   const char	*content_type;		/* CONTENT_TYPE environment variable */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
   struct sigaction action;		/* Actions for POSIX signals */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
   int		version;		/* IPP version */
-  int		reasons;		/* Number of printer-state-reasons */
   static const char * const pattrs[] =
 		{			/* Printer attributes we want */
 		  "copies-supported",
@@ -214,6 +215,9 @@ main(int  argc,				/* I - Number of command-line args */
     if ((content_type = getenv("CONTENT_TYPE")) == NULL)
       content_type = "application/octet-stream";
 
+  if (!strncmp(content_type, "printer/", 8))
+    content_type = "application/vnd.cups-raw";
+
  /*
   * Extract the hostname and printer name from the URI...
   */
@@ -571,6 +575,18 @@ main(int  argc,				/* I - Number of command-line args */
   fputs("STATE: -connecting-to-device\n", stderr);
   fprintf(stderr, "INFO: Connected to %s...\n", hostname);
 
+#ifdef AF_INET6
+  if (http->hostaddr->addr.sa_family == AF_INET6)
+    fprintf(stderr, "DEBUG: Connected to [%s]:%d (IPv6)...\n", 
+		    httpAddrString(http->hostaddr, addrname, sizeof(addrname)),
+		    ntohs(http->hostaddr->ipv6.sin6_port));
+  else
+#endif /* AF_INET6 */
+    if (http->hostaddr->addr.sa_family == AF_INET)
+      fprintf(stderr, "DEBUG: Connected to %s:%d (IPv4)...\n",
+		      httpAddrString(http->hostaddr, addrname, sizeof(addrname)),
+		      ntohs(http->hostaddr->ipv4.sin_port));
+
  /*
   * Build a URI for the printer and fill the standard IPP attributes for
   * an IPP_PRINT_FILE request.  We can't use the URI in argv[0] because it
@@ -736,19 +752,25 @@ main(int  argc,				/* I - Number of command-line args */
   * See if the printer supports multiple copies...
   */
 
+  copies = atoi(argv[4]);
+
   if (copies_sup || argc < 7)
-    copies = 1;
+  {
+    copies_remaining = 1;
+
+    if (argc < 7)
+      copies = 1;
+  }
   else
-    copies = atoi(argv[4]);
+    copies_remaining = copies;
 
  /*
   * Then issue the print-job request...
   */
 
-  reasons = 0;
   job_id  = 0;
 
-  while (copies > 0)
+  while (copies_remaining > 0)
   {
    /*
     * Build the IPP request...
@@ -775,7 +797,13 @@ main(int  argc,				/* I - Number of command-line args */
 
     fprintf(stderr, "DEBUG: requesting-user-name = \"%s\"\n", argv[2]);
 
-    if (argv[3][0])
+   /*
+    * Only add a "job-name" attribute if the remote server supports
+    * copy generation - some IPP implementations like HP's don't seem
+    * to like UTF-8 job names (STR #1837)...
+    */
+
+    if (argv[3][0] && copies_sup)
       ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "job-name", NULL,
         	   argv[3]);
 
@@ -822,8 +850,9 @@ main(int  argc,				/* I - Number of command-line args */
 	* number of copies to 1...
 	*/
 
-	content_type = "application/postscript";
-	copies       = 1;
+	content_type     = "application/postscript";
+	copies           = 1;
+	copies_remaining = 1;
       }
     }
 #endif /* __APPLE__ */
@@ -839,20 +868,21 @@ main(int  argc,				/* I - Number of command-line args */
 	                            num_options, &options);
     }
 
-    if (copies_sup)
+    if (copies_sup && version > 0)
     {
      /*
       * Only send options if the destination printer supports the copies
-      * attribute.  This is a hack for the HP JetDirect implementation of
-      * IPP, which does not accept extension attributes and incorrectly
-      * reports a client-error-bad-request error instead of the
-      * successful-ok-unsupported-attributes status.  In short, at least
-      * some HP implementations of IPP are non-compliant.
+      * attribute and IPP/1.1.  This is a hack for the HP and Lexmark
+      * implementations of IPP, which do not accept extension attributes
+      * and incorrectly report a client-error-bad-request error instead of
+      * the successful-ok-unsupported-attributes status.  In short, at least
+      * some HP and Lexmark implementations of IPP are non-compliant.
       */
 
       cupsEncodeOptions(request, num_options, options);
+
       ippAddInteger(request, IPP_TAG_JOB, IPP_TAG_INTEGER, "copies",
-                    atoi(argv[4]));
+                    copies);
     }
 
     cupsFreeOptions(num_options, options);
@@ -860,7 +890,7 @@ main(int  argc,				/* I - Number of command-line args */
    /*
     * If copies aren't supported, then we are likely dealing with an HP
     * JetDirect.  The HP IPP implementation seems to close the connection
-    * after every request (that is, it does *not* implement HTTP Keep-
+    * after every request - that is, it does *not* implement HTTP Keep-
     * Alive, which is REQUIRED by HTTP/1.1...
     */
 
@@ -891,6 +921,18 @@ main(int  argc,				/* I - Number of command-line args */
 	fputs("INFO: Printer is busy; retrying print job...\n", stderr);
 	sleep(10);
       }
+      else if ((ipp_status == IPP_BAD_REQUEST ||
+	        ipp_status == IPP_VERSION_NOT_SUPPORTED) && version == 1)
+      {
+       /*
+	* Switch to IPP/1.0...
+	*/
+
+	fputs("INFO: Printer does not support IPP/1.1, trying IPP/1.0...\n",
+	      stderr);
+	version = 0;
+	httpReconnect(http);
+      }
       else
         fprintf(stderr, "ERROR: Print file was not accepted (%s)!\n",
 	        cupsLastErrorString());
@@ -952,13 +994,13 @@ main(int  argc,				/* I - Number of command-line args */
     if (ipp_status <= IPP_OK_CONFLICT && argc > 6)
     {
       fprintf(stderr, "PAGE: 1 %d\n", copies_sup ? atoi(argv[4]) : 1);
-      copies --;
+      copies_remaining --;
     }
     else if (ipp_status == IPP_SERVICE_UNAVAILABLE ||
 	     ipp_status == IPP_PRINTER_BUSY)
       break;
     else
-      copies --;
+      copies_remaining --;
 
    /*
     * Wait for the job to complete...
@@ -1300,6 +1342,8 @@ password_cb(const char *prompt)		/* I - Prompt (not used) */
 #endif /* __APPLE__ */
 
     exit(CUPS_BACKEND_AUTH_REQUIRED);
+
+    return (NULL);			/* Eliminate compiler warning */
   }
 }
 
@@ -1622,5 +1666,5 @@ sigterm_handler(int sig)		/* I - Signal */
 
 
 /*
- * End of "$Id: ipp.c 5724 2006-07-12 19:42:35Z mike $".
+ * End of "$Id: ipp.c 6061 2006-10-23 00:26:52Z mike $".
  */
