@@ -1,5 +1,5 @@
 /*
- * "$Id: ppd.c 5333 2006-03-24 00:52:21Z mike $"
+ * "$Id: ppd.c 6036 2006-10-13 22:06:43Z mike $"
  *
  *   PPD file routines for the Common UNIX Printing System (CUPS).
  *
@@ -1549,6 +1549,18 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 	    goto error;
 
 	case 2 : /* Two options... */
+	   /*
+	    * Check for broken constraints like "* Option"...
+	    */
+
+	    if (cg->ppd_conform == PPD_CONFORM_STRICT &&
+	        (!strcmp(constraint->option1, "*") ||
+	         !strcmp(constraint->choice1, "*")))
+	    {
+	      cg->ppd_status = PPD_BAD_UI_CONSTRAINTS;
+	      goto error;
+	    }
+
 	   /*
 	    * The following strcpy's are safe, as optionN and
 	    * choiceN are all the same size (size defined by PPD spec...)
@@ -1556,17 +1568,38 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 
 	    if (constraint->option1[0] == '*')
 	      _cups_strcpy(constraint->option1, constraint->option1 + 1);
+	    else if (cg->ppd_conform == PPD_CONFORM_STRICT)
+	    {
+	      cg->ppd_status = PPD_BAD_UI_CONSTRAINTS;
+	      goto error;
+	    }
 
 	    if (constraint->choice1[0] == '*')
 	      _cups_strcpy(constraint->option2, constraint->choice1 + 1);
-	    else
-	      _cups_strcpy(constraint->option2, constraint->choice1);
+	    else if (cg->ppd_conform == PPD_CONFORM_STRICT)
+	    {
+	      cg->ppd_status = PPD_BAD_UI_CONSTRAINTS;
+	      goto error;
+	    }
 
             constraint->choice1[0] = '\0';
             constraint->choice2[0] = '\0';
 	    break;
 	    
 	case 3 : /* Two options, one choice... */
+	   /*
+	    * Check for broken constraints like "* Option"...
+	    */
+
+	    if (cg->ppd_conform == PPD_CONFORM_STRICT &&
+	        (!strcmp(constraint->option1, "*") ||
+	         !strcmp(constraint->choice1, "*") ||
+	         !strcmp(constraint->option2, "*")))
+	    {
+	      cg->ppd_status = PPD_BAD_UI_CONSTRAINTS;
+	      goto error;
+	    }
+
 	   /*
 	    * The following _cups_strcpy's are safe, as optionN and
 	    * choiceN are all the same size (size defined by PPD spec...)
@@ -1574,9 +1607,21 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 
 	    if (constraint->option1[0] == '*')
 	      _cups_strcpy(constraint->option1, constraint->option1 + 1);
+	    else if (cg->ppd_conform == PPD_CONFORM_STRICT)
+	    {
+	      cg->ppd_status = PPD_BAD_UI_CONSTRAINTS;
+	      goto error;
+	    }
 
 	    if (constraint->choice1[0] == '*')
 	    {
+	      if (cg->ppd_conform == PPD_CONFORM_STRICT &&
+	          constraint->option2[0] == '*')
+	      {
+		cg->ppd_status = PPD_BAD_UI_CONSTRAINTS;
+		goto error;
+	      }
+
 	      _cups_strcpy(constraint->choice2, constraint->option2);
 	      _cups_strcpy(constraint->option2, constraint->choice1 + 1);
               constraint->choice1[0] = '\0';
@@ -1585,20 +1630,85 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 	    {
 	      if (constraint->option2[0] == '*')
   	        _cups_strcpy(constraint->option2, constraint->option2 + 1);
+	      else if (cg->ppd_conform == PPD_CONFORM_STRICT)
+	      {
+		cg->ppd_status = PPD_BAD_UI_CONSTRAINTS;
+		goto error;
+	      }
 
               constraint->choice2[0] = '\0';
 	    }
 	    break;
 	    
 	case 4 : /* Two options, two choices... */
+	   /*
+	    * Check for broken constraints like "* Option"...
+	    */
+
+	    if (cg->ppd_conform == PPD_CONFORM_STRICT &&
+	        (!strcmp(constraint->option1, "*") ||
+	         !strcmp(constraint->choice1, "*") ||
+	         !strcmp(constraint->option2, "*") ||
+	         !strcmp(constraint->choice2, "*")))
+	    {
+	      cg->ppd_status = PPD_BAD_UI_CONSTRAINTS;
+	      goto error;
+	    }
+
 	    if (constraint->option1[0] == '*')
 	      _cups_strcpy(constraint->option1, constraint->option1 + 1);
+	    else if (cg->ppd_conform == PPD_CONFORM_STRICT)
+	    {
+	      cg->ppd_status = PPD_BAD_UI_CONSTRAINTS;
+	      goto error;
+	    }
+
+            if (cg->ppd_conform == PPD_CONFORM_STRICT &&
+	        constraint->choice1[0] == '*')
+	    {
+	      cg->ppd_status = PPD_BAD_UI_CONSTRAINTS;
+	      goto error;
+	    }
 
 	    if (constraint->option2[0] == '*')
   	      _cups_strcpy(constraint->option2, constraint->option2 + 1);
+	    else if (cg->ppd_conform == PPD_CONFORM_STRICT)
+	    {
+	      cg->ppd_status = PPD_BAD_UI_CONSTRAINTS;
+	      goto error;
+	    }
+
+            if (cg->ppd_conform == PPD_CONFORM_STRICT &&
+	        constraint->choice2[0] == '*')
+	    {
+	      cg->ppd_status = PPD_BAD_UI_CONSTRAINTS;
+	      goto error;
+	    }
 	    break;
       }
 
+     /*
+      * Handle CustomFoo option constraints...
+      */
+
+      if (!strncasecmp(constraint->option1, "Custom", 6) &&
+          !strcasecmp(constraint->choice1, "True"))
+      {
+        _cups_strcpy(constraint->option1, constraint->option1 + 6);
+	strcpy(constraint->choice1, "Custom");
+      }
+
+      if (!strncasecmp(constraint->option2, "Custom", 6) &&
+          !strcasecmp(constraint->choice2, "True"))
+      {
+        _cups_strcpy(constraint->option2, constraint->option2 + 6);
+	strcpy(constraint->choice2, "Custom");
+      }
+
+     /*
+      * Don't add this one as an attribute...
+      */
+
       ppd_free(string);
       string = NULL;
     }
@@ -2037,10 +2147,8 @@ ppd_compare_attrs(ppd_attr_t *a,	/* I - First attribute */
 
   if ((ret = strcasecmp(a->name, b->name)) != 0)
     return (ret);
-  else if (a->spec[0] && b->spec[0])
-    return (strcasecmp(a->spec, b->spec));
   else
-    return (0);
+    return (strcasecmp(a->spec, b->spec));
 }
 
 
@@ -2938,5 +3046,5 @@ ppd_read(cups_file_t    *fp,		/* I - File to read from */
 
 
 /*
- * End of "$Id: ppd.c 5333 2006-03-24 00:52:21Z mike $".
+ * End of "$Id: ppd.c 6036 2006-10-13 22:06:43Z mike $".
  */
