@@ -1,5 +1,5 @@
 /*
- * "$Id: transcode.c 5373 2006-04-06 20:03:32Z mike $"
+ * "$Id: transcode.c 6115 2006-11-15 22:21:18Z mike $"
  *
  *   Transcoding support for the Common UNIX Printing System (CUPS).
  *
@@ -295,6 +295,38 @@ cupsCharsetToUTF8(
     return (strlen((char *)dest));
   }
 
+ /*
+  * Handle ISO-8859-1 to UTF-8 directly...
+  */
+
+  if (encoding == CUPS_ISO8859_1)
+  {
+    int		ch;			/* Character from string */
+    cups_utf8_t	*destptr,		/* Pointer into UTF-8 buffer */
+		*destend;		/* End of UTF-8 buffer */
+
+
+    destptr = dest;
+    destend = dest + maxout - 2;
+
+    while (*src && destptr < destend)
+    {
+      ch = *src++ & 255;
+
+      if (ch & 128)
+      {
+	*destptr++ = 0xc0 | (ch >> 6);
+	*destptr++ = 0x80 | (ch & 0x3f);
+      }
+      else
+	*destptr++ = ch;
+    }
+
+    *destptr = '\0';
+
+    return (destptr - dest);
+  }
+
  /*
   * Convert input legacy charset to UTF-8...
   */
@@ -363,6 +395,45 @@ cupsUTF8ToCharset(
     return (strlen(dest));
   }
 
+ /*
+  * Handle UTF-8 to ISO-8859-1 directly...
+  */
+
+  if (encoding == CUPS_ISO8859_1)
+  {
+    int		ch;			/* Character from string */
+    char	*destptr,		/* Pointer into ISO-8859-1 buffer */
+		*destend;		/* End of ISO-8859-1 buffer */
+
+
+    destptr = dest;
+    destend = dest + maxout - 1;
+
+    while (*src && destptr < destend)
+    {
+      ch = *src++;
+
+      if ((ch & 0xe0) == 0xc0)
+      {
+	ch = ((ch & 0x1f) << 6) | (*src++ & 0x3f);
+
+	if (ch < 256)
+          *destptr++ = ch;
+	else
+          *destptr++ = '?';
+      }
+      else if ((ch & 0xf0) == 0xe0 ||
+               (ch & 0xf8) == 0xf0)
+        *destptr++ = '?';
+      else if (!(ch & 0x80))
+	*destptr++ = ch;
+    }
+
+    *destptr = '\0';
+
+    return (destptr - dest);
+  }
+
  /*
   * Convert input UTF-8 to legacy charset...
   */
@@ -408,7 +479,6 @@ cupsUTF8ToUTF32(
     const cups_utf8_t *src,		/* I - Source string */
     const int         maxout)		/* I - Max output */
 {
-  size_t	srclen;			/* Source string length */
   int		i;			/* Looping variable */
   cups_utf8_t	ch;			/* Character value */
   cups_utf8_t	next;			/* Next character value */
@@ -430,7 +500,6 @@ cupsUTF8ToUTF32(
   */
 
   *dest++ = 0xfeff;
-  srclen  = strlen((char *)src);
 
   for (i = maxout - 1; *src && i > 0; i --)
   {
@@ -447,6 +516,7 @@ cupsUTF8ToUTF32(
       */
 
       *dest++ = ch;
+      continue;
     }
     else if ((ch & 0xe0) == 0xc0)
     {
@@ -542,7 +612,7 @@ cupsUTF8ToUTF32(
     * Check for UTF-16 surrogate (illegal UTF-8)...
     */
 
-    if (*dest >= 0xd800 && *dest <= 0xdfff)
+    if (ch32 >= 0xd800 && ch32 <= 0xdfff)
       return (-1);
   }
 
@@ -1142,7 +1212,7 @@ free_vbcs_charmap(_cups_vmap_t *vmap)	/* I - Character set */
  */
 
 
-void *					/* O - Charset map pointer */
+static void *				/* O - Charset map pointer */
 get_charmap(
     const cups_encoding_t encoding)	/* I - Encoding */
 {
@@ -1585,5 +1655,5 @@ get_vbcs_charmap(
 
 
 /*
- * End of "$Id: transcode.c 5373 2006-04-06 20:03:32Z mike $"
+ * End of "$Id: transcode.c 6115 2006-11-15 22:21:18Z mike $"
  */
