@@ -1,9 +1,9 @@
 /*
- * "$Id: common.c,v 1.18 2002/01/23 22:30:42 mike Exp $"
+ * "$Id: common.c 6003 2006-10-02 16:26:04Z mike $"
  *
  *   Common filter routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,17 +15,21 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
+ *   This file is subject to the Apple OS-Developed Software exception.
+ *
  * Contents:
  *
  *   SetCommonOptions() - Set common filter options for media size,
  *                        etc.
  *   UpdatePageVars()   - Update the page variables for the orientation.
+ *   WriteComment()     - Write a DSC comment.
+ *   WriteCommon()      - Write common procedures...
  *   WriteLabelProlog() - Write the prolog with the classification
  *                        and page label.
  *   WriteLabels()      - Write the actual page labels.
@@ -36,38 +40,44 @@
  */
 
 #include "common.h"
+#include <locale.h>
 
 
 /*
  * Globals...
  */
 
-int	Orientation = 0,	/* 0 = portrait, 1 = landscape, etc. */
-	Duplex = 0,		/* Duplexed? */
-	LanguageLevel = 1,	/* Language level of printer */
-	ColorDevice = 1;	/* Do color text? */
-float	PageLeft = 18.0f,	/* Left margin */
-	PageRight = 594.0f,	/* Right margin */
-	PageBottom = 36.0f,	/* Bottom margin */
-	PageTop = 756.0f,	/* Top margin */
-	PageWidth = 612.0f,	/* Total page width */
-	PageLength = 792.0f;	/* Total page length */
+int	Orientation = 0,		/* 0 = portrait, 1 = landscape, etc. */
+	Duplex = 0,			/* Duplexed? */
+	LanguageLevel = 1,		/* Language level of printer */
+	ColorDevice = 1;		/* Do color text? */
+float	PageLeft = 18.0f,		/* Left margin */
+	PageRight = 594.0f,		/* Right margin */
+	PageBottom = 36.0f,		/* Bottom margin */
+	PageTop = 756.0f,		/* Top margin */
+	PageWidth = 612.0f,		/* Total page width */
+	PageLength = 792.0f;		/* Total page length */
 
 
 /*
  * 'SetCommonOptions()' - Set common filter options for media size, etc.
  */
 
-ppd_file_t *					/* O - PPD file */
-SetCommonOptions(int           num_options,	/* I - Number of options */
-                 cups_option_t *options,	/* I - Options */
-		 int           change_size)	/* I - Change page size? */
+ppd_file_t *				/* O - PPD file */
+SetCommonOptions(
+    int           num_options,		/* I - Number of options */
+    cups_option_t *options,		/* I - Options */
+    int           change_size)		/* I - Change page size? */
 {
-  ppd_file_t	*ppd;		/* PPD file */
-  ppd_size_t	*pagesize;	/* Current page size */
-  const char	*val;		/* Option value */
+  ppd_file_t	*ppd;			/* PPD file */
+  ppd_size_t	*pagesize;		/* Current page size */
+  const char	*val;			/* Option value */
 
 
+#ifdef LC_TIME
+  setlocale(LC_TIME, "");
+#endif /* LC_TIME */
+
   ppd = ppdOpenFile(getenv("PPD"));
 
   ppdMarkDefaults(ppd);
@@ -93,9 +103,17 @@ SetCommonOptions(int           num_options,	/* I - Number of options */
   }
 
   if ((val = cupsGetOption("landscape", num_options, options)) != NULL)
-    Orientation = 1;
-
-  if ((val = cupsGetOption("orientation-requested", num_options, options)) != NULL)
+  {
+    if (strcasecmp(val, "no") != 0 && strcasecmp(val, "off") != 0 &&
+        strcasecmp(val, "false") != 0)
+    {
+      if (ppd && ppd->landscape > 0)
+        Orientation = 1;
+      else
+        Orientation = 3;
+    }
+  }
+  else if ((val = cupsGetOption("orientation-requested", num_options, options)) != NULL)
   {
    /*
     * Map IPP orientation values to 0 to 3:
@@ -190,14 +208,14 @@ SetCommonOptions(int           num_options,	/* I - Number of options */
   if (change_size)
     UpdatePageVars();
 
-  if ((val = cupsGetOption("sides", num_options, options)) != NULL &&
-      strncasecmp(val, "two-", 4) == 0)
-    Duplex = 1;
-  else if ((val = cupsGetOption("Duplex", num_options, options)) != NULL &&
-           strncasecmp(val, "Duplex", 6) == 0)
-    Duplex = 1;
-  else if (ppdIsMarked(ppd, "Duplex", "DuplexNoTumble") ||
-           ppdIsMarked(ppd, "Duplex", "DuplexTumble"))
+  if (ppdIsMarked(ppd, "Duplex", "DuplexNoTumble") ||
+      ppdIsMarked(ppd, "Duplex", "DuplexTumble") ||
+      ppdIsMarked(ppd, "JCLDuplex", "DuplexNoTumble") ||
+      ppdIsMarked(ppd, "JCLDuplex", "DuplexTumble") ||
+      ppdIsMarked(ppd, "EFDuplex", "DuplexNoTumble") ||
+      ppdIsMarked(ppd, "EFDuplex", "DuplexTumble") ||
+      ppdIsMarked(ppd, "KD03Duplex", "DuplexNoTumble") ||
+      ppdIsMarked(ppd, "KD03Duplex", "DuplexTumble"))
     Duplex = 1;
 
   return (ppd);
@@ -211,7 +229,7 @@ SetCommonOptions(int           num_options,	/* I - Number of options */
 void
 UpdatePageVars(void)
 {
-  float		temp;		/* Swapping variable */
+  float		temp;			/* Swapping variable */
 
 
   switch (Orientation & 3)
@@ -268,6 +286,28 @@ UpdatePageVars(void)
 }
 
 
+/*
+ * 'WriteCommon()' - Write common procedures...
+ */
+
+void
+WriteCommon(void)
+{
+  puts("% x y w h ESPrc - Clip to a rectangle.\n"
+       "userdict/ESPrc/rectclip where{pop/rectclip load}\n"
+       "{{newpath 4 2 roll moveto 1 index 0 rlineto 0 exch rlineto\n"
+       "neg 0 rlineto closepath clip newpath}bind}ifelse put");
+  puts("% x y w h ESPrf - Fill a rectangle.\n"
+       "userdict/ESPrf/rectfill where{pop/rectfill load}\n"
+       "{{gsave newpath 4 2 roll moveto 1 index 0 rlineto 0 exch rlineto\n"
+       "neg 0 rlineto closepath fill grestore}bind}ifelse put");
+  puts("% x y w h ESPrs - Stroke a rectangle.\n"
+       "userdict/ESPrs/rectstroke where{pop/rectstroke load}\n"
+       "{{gsave newpath 4 2 roll moveto 1 index 0 rlineto 0 exch rlineto\n"
+       "neg 0 rlineto closepath stroke grestore}bind}ifelse put");
+}
+
+
 /*
  * 'WriteLabelProlog()' - Write the prolog with the classification
  *                        and page label.
@@ -280,6 +320,7 @@ WriteLabelProlog(const char *label,	/* I - Page label */
 		 float      width)	/* I - Width in points */
 {
   const char	*classification;	/* CLASSIFICATION environment variable */
+  const char	*ptr;			/* Temporary string pointer */
 
 
  /*
@@ -298,7 +339,7 @@ WriteLabelProlog(const char *label,	/* I - Page label */
 
   if (!classification[0] && (label == NULL || !label[0]))
   {
-    puts("/espWL{}bind def");
+    puts("userdict/ESPwl{}bind put");
     return;
   }
 
@@ -306,50 +347,82 @@ WriteLabelProlog(const char *label,	/* I - Page label */
   * Set the classification + page label string...
   */
 
+  printf("userdict");
   if (strcmp(classification, "confidential") == 0)
-    printf("/espPL(CONFIDENTIAL");
+    printf("/ESPpl(CONFIDENTIAL");
   else if (strcmp(classification, "classified") == 0)
-    printf("/espPL(CLASSIFIED");
+    printf("/ESPpl(CLASSIFIED");
   else if (strcmp(classification, "secret") == 0)
-    printf("/espPL(SECRET");
+    printf("/ESPpl(SECRET");
   else if (strcmp(classification, "topsecret") == 0)
-    printf("/espPL(TOP SECRET");
+    printf("/ESPpl(TOP SECRET");
   else if (strcmp(classification, "unclassified") == 0)
-    printf("/espPL(UNCLASSIFIED");
+    printf("/ESPpl(UNCLASSIFIED");
   else
-    printf("/espPL(");
+  {
+    printf("/ESPpl(");
 
-  if (classification[0] && label)
-    printf(" - %s)def\n", label);
-  else if (label)
-    printf("%s)def\n", label);
-  else
-    puts(")def");
+    for (ptr = classification; *ptr; ptr ++)
+      if (*ptr < 32 || *ptr > 126)
+        printf("\\%03o", *ptr);
+      else if (*ptr == '_')
+        putchar(' ');
+      else
+      {
+	if (*ptr == '(' || *ptr == ')' || *ptr == '\\')
+	  putchar('\\');
+
+	putchar(*ptr);
+      }
+  }
+
+  if (label)
+  {
+    if (classification[0])
+      printf(" - ");
+
+   /*
+    * Quote the label string as needed...
+    */
+
+    for (ptr = label; *ptr; ptr ++)
+      if (*ptr < 32 || *ptr > 126)
+        printf("\\%03o", *ptr);
+      else
+      {
+	if (*ptr == '(' || *ptr == ')' || *ptr == '\\')
+	  putchar('\\');
+
+	putchar(*ptr);
+      }
+  }
+
+  puts(")put");
 
  /*
   * Then get a 14 point Helvetica-Bold font...
   */
 
-  puts("/espPF /Helvetica-Bold findfont 14 scalefont def");
+  puts("userdict/ESPpf /Helvetica-Bold findfont 14 scalefont put");
 
  /*
   * Finally, the procedure to write the labels on the page...
   */
 
-  puts("/espWL{");
-  puts("  espPF setfont");
-  printf("  espPL stringwidth pop dup 12 add exch  -0.5 mul %.0f add\n",
+  puts("userdict/ESPwl{");
+  puts("  ESPpf setfont");
+  printf("  ESPpl stringwidth pop dup 12 add exch -0.5 mul %.0f add\n",
          width * 0.5f);
   puts("  1 setgray");
-  printf("  dup 6 sub %.0f 3 index 20 rectfill\n", bottom - 2.0);
-  printf("  dup 6 sub %.0f 3 index 20 rectfill\n", top - 18.0);
+  printf("  dup 6 sub %.0f 3 index 20 ESPrf\n", bottom - 2.0);
+  printf("  dup 6 sub %.0f 3 index 20 ESPrf\n", top - 18.0);
   puts("  0 setgray");
-  printf("  dup 6 sub %.0f 3 index 20 rectstroke\n", bottom - 2.0);
-  printf("  dup 6 sub %.0f 3 index 20 rectstroke\n", top - 18.0);
-  printf("  dup %.0f moveto espPL show\n", bottom + 2.0);
-  printf("  %.0f moveto espPL show\n", top - 14.0);
+  printf("  dup 6 sub %.0f 3 index 20 ESPrs\n", bottom - 2.0);
+  printf("  dup 6 sub %.0f 3 index 20 ESPrs\n", top - 18.0);
+  printf("  dup %.0f moveto ESPpl show\n", bottom + 2.0);
+  printf("  %.0f moveto ESPpl show\n", top - 14.0);
   puts("pop");
-  puts("}bind def");
+  puts("}bind put");
 }
 
 
@@ -390,11 +463,82 @@ WriteLabels(int orient)	/* I - Orientation of the page */
         break;
   }
 
-  puts("espWL");
+  puts("ESPwl");
   puts("grestore");
 }
 
 
 /*
- * End of "$Id: common.c,v 1.18 2002/01/23 22:30:42 mike Exp $".
+ * 'WriteTextComment()' - Write a DSC text comment.
+ */
+
+void
+WriteTextComment(const char *name,	/* I - Comment name ("Title", etc.) */
+                 const char *value)	/* I - Comment value */
+{
+  int	len;				/* Current line length */
+
+
+ /*
+  * DSC comments are of the form:
+  *
+  *   %%name: value
+  *
+  * The name and value must be limited to 7-bit ASCII for most printers,
+  * so we escape all non-ASCII and ASCII control characters as described
+  * in the Adobe Document Structuring Conventions specification.
+  */
+
+  printf("%%%%%s: (", name);
+  len = 5 + strlen(name);
+
+  while (*value)
+  {
+    if (*value < ' ' || *value >= 127)
+    {
+     /*
+      * Escape this character value...
+      */
+
+      if (len >= 251)			/* Keep line < 254 chars */
+        break;
+
+      printf("\\%03o", *value & 255);
+      len += 4;
+    }
+    else if (*value == '\\')
+    {
+     /*
+      * Escape the backslash...
+      */
+
+      if (len >= 253)			/* Keep line < 254 chars */
+        break;
+
+      putchar('\\');
+      putchar('\\');
+      len += 2;
+    }
+    else
+    {
+     /*
+      * Put this character literally...
+      */
+
+      if (len >= 254)			/* Keep line < 254 chars */
+        break;
+
+      putchar(*value);
+      len ++;
+    }
+
+    value ++;
+  }
+
+  puts(")");
+}
+
+
+/*
+ * End of "$Id: common.c 6003 2006-10-02 16:26:04Z mike $".
  */
