@@ -1,9 +1,9 @@
 /*
- * "$Id: image-pnm.c,v 1.8 2002/01/02 17:58:59 mike Exp $"
+ * "$Id: image-pnm.c 5509 2006-05-11 11:41:36Z mike $"
  *
  *   Portable Any Map file routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 1993-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,50 +15,52 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
+ *   This file is subject to the Apple OS-Developed Software exception.
+ *
  * Contents:
  *
- *   ImageReadPNM() - Read a PNM image file.
+ *   _cupsImageReadPNM() - Read a PNM image file.
  */
 
 /*
  * Include necessary headers...
  */
 
-#include "image.h"
-#include <ctype.h>
+#include "image-private.h"
 
 
 /*
- * 'ImageReadPNM()' - Read a PNM image file.
+ * '_cupsImageReadPNM()' - Read a PNM image file.
  */
 
 int					/* O - Read status */
-ImageReadPNM(image_t    *img,		/* IO - Image */
-             FILE       *fp,		/* I - Image file */
-             int        primary,	/* I - Primary choice for colorspace */
-             int        secondary,	/* I - Secondary choice for colorspace */
-             int        saturation,	/* I - Color saturation (%) */
-             int        hue,		/* I - Color hue (degrees) */
-	     const ib_t *lut)		/* I - Lookup table for gamma/brightness */
+_cupsImageReadPNM(
+    cups_image_t    *img,		/* IO - cupsImage */
+    FILE            *fp,		/* I - cupsImage file */
+    cups_icspace_t  primary,		/* I - Primary choice for colorspace */
+    cups_icspace_t  secondary,		/* I - Secondary choice for colorspace */
+    int             saturation,		/* I - Color saturation (%) */
+    int             hue,		/* I - Color hue (degrees) */
+    const cups_ib_t *lut)		/* I - Lookup table for gamma/brightness */
 {
-  int		x, y;		/* Looping vars */
-  int		bpp;		/* Bytes per pixel */
-  ib_t		*in,		/* Input pixels */
-		*inptr,		/* Current input pixel */
-		*out,		/* Output pixels */
-		*outptr,	/* Current output pixel */
-		bit;		/* Bit in input line */
-  char		line[255],	/* Input line */
-		*lineptr;	/* Pointer in line */
-  int		format,		/* Format of PNM file */
-		val,		/* Pixel value */
-		maxval;		/* Maximum pixel value */
+  int		x, y;			/* Looping vars */
+  int		bpp;			/* Bytes per pixel */
+  cups_ib_t	*in,			/* Input pixels */
+		*inptr,			/* Current input pixel */
+		*out,			/* Output pixels */
+		*outptr,		/* Current output pixel */
+		bit;			/* Bit in input line */
+  char		line[255],		/* Input line */
+		*lineptr;		/* Pointer in line */
+  int		format,			/* Format of PNM file */
+		val,			/* Pixel value */
+		maxval;			/* Maximum pixel value */
 
 
  /*
@@ -78,17 +80,17 @@ ImageReadPNM(image_t    *img,		/* IO - Image */
   lineptr ++;
 
   format = atoi(lineptr);
-  while (isdigit(*lineptr))
+  while (isdigit(*lineptr & 255))
     lineptr ++;
 
   while (lineptr != NULL && img->xsize == 0)
   {
     if (*lineptr == '\0' || *lineptr == '#')
       lineptr = fgets(line, sizeof(line), fp);
-    else if (isdigit(*lineptr))
+    else if (isdigit(*lineptr & 255))
     {
       img->xsize = atoi(lineptr);
-      while (isdigit(*lineptr))
+      while (isdigit(*lineptr & 255))
 	lineptr ++;
     }
     else
@@ -99,10 +101,10 @@ ImageReadPNM(image_t    *img,		/* IO - Image */
   {
     if (*lineptr == '\0' || *lineptr == '#')
       lineptr = fgets(line, sizeof(line), fp);
-    else if (isdigit(*lineptr))
+    else if (isdigit(*lineptr & 255))
     {
       img->ysize = atoi(lineptr);
-      while (isdigit(*lineptr))
+      while (isdigit(*lineptr & 255))
 	lineptr ++;
     }
     else
@@ -117,10 +119,10 @@ ImageReadPNM(image_t    *img,		/* IO - Image */
     {
       if (*lineptr == '\0' || *lineptr == '#')
 	lineptr = fgets(line, sizeof(line), fp);
-      else if (isdigit(*lineptr))
+      else if (isdigit(*lineptr & 255))
       {
 	maxval = atoi(lineptr);
-	while (isdigit(*lineptr))
+	while (isdigit(*lineptr & 255))
 	  lineptr ++;
       }
       else
@@ -130,14 +132,30 @@ ImageReadPNM(image_t    *img,		/* IO - Image */
   else
     maxval = 1;
 
+  if (img->xsize == 0 || img->xsize > CUPS_IMAGE_MAX_WIDTH ||
+      img->ysize == 0 || img->ysize > CUPS_IMAGE_MAX_HEIGHT)
+  {
+    fprintf(stderr, "ERROR: Bad PNM dimensions %dx%d!\n",
+            img->xsize, img->ysize);
+    fclose(fp);
+    return (1);
+  }
+
+  if (maxval == 0)
+  {
+    fprintf(stderr, "ERROR: Bad PNM max value %d!\n", maxval);
+    fclose(fp);
+    return (1);
+  }
+
   if (format == 1 || format == 2 || format == 4 || format == 5)
     img->colorspace = secondary;
   else
-    img->colorspace = primary;
+    img->colorspace = (primary == CUPS_IMAGE_RGB_CMYK) ? CUPS_IMAGE_RGB : primary;
 
-  ImageSetMaxTiles(img, 0);
+  cupsImageSetMaxTiles(img, 0);
 
-  bpp = ImageGetDepth(img);
+  bpp = cupsImageGetDepth(img);
   in  = malloc(img->xsize * 3);
   out = malloc(img->xsize * bpp);
 
@@ -204,72 +222,71 @@ ImageReadPNM(image_t    *img,		/* IO - Image */
       case 2 :
       case 4 :
       case 5 :
-          if (img->colorspace == IMAGE_WHITE)
+          if (img->colorspace == CUPS_IMAGE_WHITE)
 	  {
 	    if (lut)
-	      ImageLut(in, img->xsize, lut);
+	      cupsImageLut(in, img->xsize, lut);
 
-            ImagePutRow(img, 0, y, img->xsize, in);
+            _cupsImagePutRow(img, 0, y, img->xsize, in);
 	  }
 	  else
 	  {
 	    switch (img->colorspace)
 	    {
-	      case IMAGE_RGB :
-		  ImageWhiteToRGB(in, out, img->xsize);
+              default :
 		  break;
-	      case IMAGE_BLACK :
-		  ImageWhiteToBlack(in, out, img->xsize);
+
+	      case CUPS_IMAGE_RGB :
+		  cupsImageWhiteToRGB(in, out, img->xsize);
 		  break;
-	      case IMAGE_CMY :
-		  ImageWhiteToCMY(in, out, img->xsize);
+	      case CUPS_IMAGE_BLACK :
+		  cupsImageWhiteToBlack(in, out, img->xsize);
 		  break;
-	      case IMAGE_CMYK :
-		  ImageWhiteToCMYK(in, out, img->xsize);
+	      case CUPS_IMAGE_CMY :
+		  cupsImageWhiteToCMY(in, out, img->xsize);
+		  break;
+	      case CUPS_IMAGE_CMYK :
+		  cupsImageWhiteToCMYK(in, out, img->xsize);
 		  break;
 	    }
 
 	    if (lut)
-	      ImageLut(out, img->xsize * bpp, lut);
+	      cupsImageLut(out, img->xsize * bpp, lut);
 
-            ImagePutRow(img, 0, y, img->xsize, out);
+            _cupsImagePutRow(img, 0, y, img->xsize, out);
 	  }
 	  break;
 
       default :
 	  if ((saturation != 100 || hue != 0) && bpp > 1)
-	    ImageRGBAdjust(in, img->xsize, saturation, hue);
+	    cupsImageRGBAdjust(in, img->xsize, saturation, hue);
 
-	  if (img->colorspace == IMAGE_RGB)
+	  switch (img->colorspace)
 	  {
-	    if (lut)
-	      ImageLut(in, img->xsize * 3, lut);
+            default :
+		break;
 
-            ImagePutRow(img, 0, y, img->xsize, in);
+	    case CUPS_IMAGE_WHITE :
+		cupsImageRGBToWhite(in, out, img->xsize);
+		break;
+	    case CUPS_IMAGE_RGB :
+		cupsImageRGBToRGB(in, out, img->xsize);
+		break;
+	    case CUPS_IMAGE_BLACK :
+		cupsImageRGBToBlack(in, out, img->xsize);
+		break;
+	    case CUPS_IMAGE_CMY :
+		cupsImageRGBToCMY(in, out, img->xsize);
+		break;
+	    case CUPS_IMAGE_CMYK :
+		cupsImageRGBToCMYK(in, out, img->xsize);
+		break;
 	  }
-	  else
-	  {
-	    switch (img->colorspace)
-	    {
-	      case IMAGE_WHITE :
-		  ImageRGBToWhite(in, out, img->xsize);
-		  break;
-	      case IMAGE_BLACK :
-		  ImageRGBToBlack(in, out, img->xsize);
-		  break;
-	      case IMAGE_CMY :
-		  ImageRGBToCMY(in, out, img->xsize);
-		  break;
-	      case IMAGE_CMYK :
-		  ImageRGBToCMYK(in, out, img->xsize);
-		  break;
-	    }
 
-	    if (lut)
-	      ImageLut(out, img->xsize * bpp, lut);
+	  if (lut)
+	    cupsImageLut(out, img->xsize * bpp, lut);
 
-            ImagePutRow(img, 0, y, img->xsize, out);
-	  }
+          _cupsImagePutRow(img, 0, y, img->xsize, out);
   	  break;
     }
   }
@@ -284,5 +301,5 @@ ImageReadPNM(image_t    *img,		/* IO - Image */
 
 
 /*
- * End of "$Id: image-pnm.c,v 1.8 2002/01/02 17:58:59 mike Exp $".
+ * End of "$Id: image-pnm.c 5509 2006-05-11 11:41:36Z mike $".
  */
