@@ -1,10 +1,10 @@
 /*
- * "$Id: image-sgi.h,v 1.6 2002/01/02 17:58:59 mike Exp $"
+ * "$Id: image-sgi.h 4741 2005-10-02 04:25:52Z mike $"
  *
  *   SGI image file format library definitions for the Common UNIX Printing
  *   System (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 1993-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -16,11 +16,13 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
+ *
+ *   This file is subject to the Apple OS-Developed Software exception.
  */
 
 #ifndef _SGI_H_
@@ -78,7 +80,7 @@ typedef struct
 
 extern int	sgiClose(sgi_t *sgip);
 extern int	sgiGetRow(sgi_t *sgip, unsigned short *row, int y, int z);
-extern sgi_t	*sgiOpen(char *filename, int mode, int comp, int bpp,
+extern sgi_t	*sgiOpen(const char *filename, int mode, int comp, int bpp,
 		         int xsize, int ysize, int zsize);
 extern sgi_t	*sgiOpenFile(FILE *file, int mode, int comp, int bpp,
 		             int xsize, int ysize, int zsize);
@@ -90,5 +92,5 @@ extern int	sgiPutRow(sgi_t *sgip, unsigned short *row, int y, int z);
 #endif /* !_SGI_H_ */
 
 /*
- * End of "$Id: image-sgi.h,v 1.6 2002/01/02 17:58:59 mike Exp $".
+ * End of "$Id: image-sgi.h 4741 2005-10-02 04:25:52Z mike $".
  */
