@@ -1,5 +1,5 @@
 /*
- * "$Id: raster.h 5485 2006-05-02 23:59:56Z mike $"
+ * "$Id: raster.h 6061 2006-10-23 00:26:52Z mike $"
  *
  *   Raster file definitions for the Common UNIX Printing System (CUPS).
  *
@@ -23,13 +23,6 @@
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
- *   This code and any derivative of it may be used and distributed
- *   freely under the terms of the GNU General Public License when
- *   used with GNU/GPL Ghostscript or its derivatives.  Use of the
- *   code (or any derivative of it) with software other than GNU/GPL
- *   GhostScript (or its derivatives) is governed by the CUPS license
- *   agreement.
- *
  *   This file is subject to the Apple OS-Developed Software exception.
  */
 
@@ -60,12 +53,15 @@ extern "C" {
  * Constants...
  */
 
-#  define CUPS_RASTER_SYNC	0x52615332	/* RaS2 */
-#  define CUPS_RASTER_REVSYNC	0x32536152	/* 2SaR */
+#  define CUPS_RASTER_SYNC	0x52615333	/* RaS3 */
+#  define CUPS_RASTER_REVSYNC	0x33536152	/* 3SaR */
 
 #  define CUPS_RASTER_SYNCv1	0x52615374	/* RaSt */
 #  define CUPS_RASTER_REVSYNCv1	0x74536152	/* tSaR */
 
+#  define CUPS_RASTER_SYNCv2	0x52615332	/* RaS2 */
+#  define CUPS_RASTER_REVSYNCv2	0x32536152	/* 2SaR */
+
 
 /*
  * The following definition can be used to determine if the
@@ -303,19 +299,8 @@ typedef struct cups_page_header2_s	/**** Version 2 Page Header @since CUPS 1.2@
   char		cupsPageSizeName[64];	/* PageSize name @since CUPS 1.2@ */
 } cups_page_header2_t;
 
-typedef struct _cups_raster_s		/**** Raster stream data ****/
-{
-  unsigned		sync;		/* Sync word from start of stream */
-  int			fd;		/* File descriptor */
-  cups_mode_t		mode;		/* Read/write mode */
-  cups_page_header2_t	header;		/* Raster header for current page */
-  int			count,		/* Current row run-length count */
-			remaining,	/* Remaining rows in page image */
-			bpp;		/* Bytes per pixel/color */
-  unsigned char		*pixels,	/* Pixels for current row */
-			*pend,		/* End of pixel buffer */
-			*pcurrent;	/* Current byte in pixel buffer */
-} cups_raster_t;
+typedef struct _cups_raster_s cups_raster_t;
+					/**** Raster stream data ****/
 
 typedef int (*cups_interpret_cb_t)(cups_page_header2_t *header, int preferred_bits);
 
@@ -353,5 +338,5 @@ extern unsigned		cupsRasterWriteHeader2(cups_raster_t *r,
 #endif /* !_CUPS_RASTER_H_ */
 
 /*
- * End of "$Id: raster.h 5485 2006-05-02 23:59:56Z mike $".
+ * End of "$Id: raster.h 6061 2006-10-23 00:26:52Z mike $".
  */
