@@ -1,10 +1,10 @@
 /*
- * "$Id: rastertoepson.c,v 1.14 2002/01/02 17:59:01 mike Exp $"
+ * "$Id: rastertoepson.c 4494 2005-02-18 02:18:11Z mike $"
  *
  *   EPSON ESC/P and ESC/P2 filter for the Common UNIX Printing System
  *   (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 1993-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -16,12 +16,14 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
+ *   This file is subject to the Apple OS-Developed Software exception.
+ *
  * Contents:
  *
  *   Setup()        - Prepare the printer for printing.
@@ -75,7 +77,8 @@ unsigned char	*Planes[6],		/* Output buffers */
 		*LineBuffers[2];	/* Line bitmap buffers */
 int		Model,			/* Model number */
 		NumPlanes,		/* Number of color planes */
-		Feed;			/* Number of lines to skip */
+		Feed,			/* Number of lines to skip */
+		EjectPage;		/* Eject the page when done? */
 int		DotBit,			/* Bit in buffers */
 		DotBytes,		/* # bytes in a dot column */
 		DotColumns,		/* # columns in 1/60 inch */
@@ -117,7 +120,7 @@ Setup(void)
   */
 
   if ((device_uri = getenv("DEVICE_URI")) != NULL &&
-      strncmp(device_uri, "usb:", 4) == 0)
+      strncmp(device_uri, "usb:", 4) == 0 && Model >= EPSON_ICOLOR)
     pwrite("\000\000\000\033\001@EJL 1284.4\n@EJL     \n\033@", 29);
 }
 
@@ -167,13 +170,13 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
   * See which type of printer we are using...
   */
 
-  Model = ppd->model_number;
-
+  EjectPage = header->Margins[0] || header->Margins[1];
+    
   switch (ppd->model_number)
   {
     case EPSON_9PIN :
     case EPSON_24PIN :
-        printf("\033P");		/* Set 10 CPI */
+        printf("\033P\022");		/* Set 10 CPI */
 
 	if (header->HWResolution[0] == 360 || header->HWResolution[0] == 240)
 	{
@@ -191,6 +194,7 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
 	printf("\033C%c%c", 0,		/* Page length */
                       (int)(header->PageSize[1] / 72.0 + 0.5));
 	printf("\033N%c", 0);		/* Bottom margin */
+        printf("\033O");		/* No perforation skip */
 
        /*
 	* Setup various buffer limits...
@@ -207,6 +211,7 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
 	  {
 	    case 60:
 	    case 120 :
+	    case 240 :
         	printf("\033\063\030");	/* Set line feed */
 		break;
 
@@ -336,7 +341,10 @@ EndPage(const cups_page_header_t *header)	/* I - Page header */
     */
 
     if (!Shingling)
-      OutputRows(header, 0);
+    {
+      if (DotBit < 128 || EvenOffset)
+        OutputRows(header, 0);
+    }
     else if (OddOffset > EvenOffset)
     {
       OutputRows(header, 1);
@@ -353,7 +361,8 @@ EndPage(const cups_page_header_t *header)	/* I - Page header */
   * Eject the current page...
   */
 
-  putchar(12);		/* Form feed */
+  if (EjectPage)
+    putchar(12);		/* Form feed */
   fflush(stdout);
 
  /*
@@ -726,7 +735,7 @@ OutputLine(const cups_page_header_t *header)	/* I - Page header */
 
         for (width = header->cupsWidth, tempptr = CompBuffer,
                  evenptr = LineBuffers[0] + EvenOffset;
-             width >= 0;
+             width > 0;
              width --, tempptr ++, evenptr += DotBytes)
           *evenptr = tempptr[0];
       }
@@ -862,10 +871,15 @@ OutputRows(const cups_page_header_t *header,	/* I - Page image header */
     * Position print head for printing...
     */
 
-    putchar(0x1b);
-    putchar('$');
-    putchar(i & 255);
-    putchar(i >> 8);
+    if (i == 0)
+      putchar('\r');
+    else
+    {
+      putchar(0x1b);
+      putchar('$');
+      putchar(i & 255);
+      putchar(i >> 8);
+    }
 
    /*
     * Start bitmap graphics for this line...
@@ -929,10 +943,15 @@ OutputRows(const cups_page_header_t *header,	/* I - Page image header */
       * Move the head back and print the odd bytes...
       */
 
-      putchar(0x1b);
-      putchar('$');
-      putchar(i & 255);
-      putchar(i >> 8);
+      if (i == 0)
+	putchar('\r');
+      else
+      {
+	putchar(0x1b);
+	putchar('$');
+	putchar(i & 255);
+	putchar(i >> 8);
+      }
 
       if (header->HWResolution[0] == 120)
       	printf("\033*\001");		/* Select bit image */
@@ -1037,6 +1056,8 @@ main(int  argc,			/* I - Number of command-line arguments */
   */
 
   ppd = ppdOpenFile(getenv("PPD"));
+  if (ppd)
+    Model = ppd->model_number;
 
   Setup();
 
@@ -1127,5 +1148,5 @@ main(int  argc,			/* I - Number of command-line arguments */
 
 
 /*
- * End of "$Id: rastertoepson.c,v 1.14 2002/01/02 17:59:01 mike Exp $".
+ * End of "$Id: rastertoepson.c 4494 2005-02-18 02:18:11Z mike $".
  */
