@@ -1,5 +1,5 @@
 /*
- * "$Id: textcommon.c,v 1.33 2005/01/03 19:29:58 mike Exp $"
+ * "$Id: textcommon.c 4559 2005-08-04 18:40:13Z mike $"
  *
  *   Common text filter routines for the Common UNIX Printing System (CUPS).
  *
@@ -568,9 +568,9 @@ TextMain(const char *name,	/* I - Name of filter */
 
     if ((val = getenv("CONTENT_TYPE")) == NULL)
     {
-      PrettyPrint = PRETTY_CODE;
-      NumKeywords = sizeof(code_keywords) / sizeof(code_keywords[0]);
-      Keywords    = code_keywords;
+      PrettyPrint = PRETTY_PLAIN;
+      NumKeywords = 0;
+      Keywords    = NULL;
     }
     else if (strcasecmp(val, "application/x-cshell") == 0)
     {
@@ -578,6 +578,12 @@ TextMain(const char *name,	/* I - Name of filter */
       NumKeywords = sizeof(csh_keywords) / sizeof(csh_keywords[0]);
       Keywords    = csh_keywords;
     }
+    else if (strcasecmp(val, "application/x-csource") == 0)
+    {
+      PrettyPrint = PRETTY_CODE;
+      NumKeywords = sizeof(code_keywords) / sizeof(code_keywords[0]);
+      Keywords    = code_keywords;
+    }
     else if (strcasecmp(val, "application/x-perl") == 0)
     {
       PrettyPrint = PRETTY_PERL;
@@ -592,9 +598,9 @@ TextMain(const char *name,	/* I - Name of filter */
     }
     else
     {
-      PrettyPrint = PRETTY_CODE;
-      NumKeywords = sizeof(code_keywords) / sizeof(code_keywords[0]);
-      Keywords    = code_keywords;
+      PrettyPrint = PRETTY_PLAIN;
+      NumKeywords = 0;
+      Keywords    = NULL;
     }
   }
 
@@ -707,6 +713,8 @@ TextMain(const char *name,	/* I - Name of filter */
           }
 
 	  keycol = column;
+
+          attr &= ~ATTR_BOLD;
           break;
 
       case 0x0d :		/* CR */
@@ -874,14 +882,14 @@ TextMain(const char *name,	/* I - Name of filter */
           if (ch < ' ')
             break;		/* Ignore other control chars */
 
-          if (PrettyPrint)
+          if (PrettyPrint > PRETTY_PLAIN)
 	  {
 	   /*
 	    * Do highlighting of C/C++ keywords, preprocessor commands,
 	    * and comments...
 	    */
 
-	    if ((ch == ' ' || ch == '\t') && (attr & ATTR_BOLD))
+	    if (ch == ' ' && (attr & ATTR_BOLD))
 	    {
 	     /*
 	      * Stop bolding preprocessor command...
@@ -934,7 +942,7 @@ TextMain(const char *name,	/* I - Name of filter */
 	      */
 
 	      cstring = -1;
-              attr    |= ATTR_BLUE;
+              attr    = ATTR_BLUE;
 	    }
             else if (ch == '*' && lastch == '/' && !cstring &&
 	             PrettyPrint != PRETTY_SHELL)
@@ -944,7 +952,7 @@ TextMain(const char *name,	/* I - Name of filter */
 	      */
 
 	      ccomment = 1;
-	      attr     |= ATTR_ITALIC | ATTR_GREEN;
+	      attr     = ATTR_ITALIC | ATTR_GREEN;
 	    }
 	    else if (ch == '/' && lastch == '/' && !cstring &&
 	             PrettyPrint == PRETTY_CODE)
@@ -953,7 +961,7 @@ TextMain(const char *name,	/* I - Name of filter */
 	      * Start a C++-style comment...
 	      */
 
-	      attr |= ATTR_ITALIC | ATTR_GREEN;
+	      attr = ATTR_ITALIC | ATTR_GREEN;
 	    }
 	    else if (ch == '#' && !cstring && PrettyPrint != PRETTY_CODE)
 	    {
@@ -961,7 +969,7 @@ TextMain(const char *name,	/* I - Name of filter */
 	      * Start a shell-style comment...
 	      */
 
-	      attr |= ATTR_ITALIC | ATTR_GREEN;
+	      attr = ATTR_ITALIC | ATTR_GREEN;
 	    }
 	    else if (ch == '#' && column == 0 && !ccomment && !cstring &&
 	             PrettyPrint == PRETTY_CODE)
@@ -970,7 +978,7 @@ TextMain(const char *name,	/* I - Name of filter */
 	      * Start a preprocessor command...
 	      */
 
-	      attr |= ATTR_BOLD | ATTR_RED;
+	      attr = ATTR_BOLD | ATTR_RED;
 	    }
           }
 
@@ -1183,5 +1191,5 @@ getutf8(FILE *fp)	/* I - File to read from */
 
 
 /*
- * End of "$Id: textcommon.c,v 1.33 2005/01/03 19:29:58 mike Exp $".
+ * End of "$Id: textcommon.c 4559 2005-08-04 18:40:13Z mike $".
  */
