@@ -1,9 +1,9 @@
 /*
- * "$Id: texttops.c,v 1.40 2002/01/30 16:32:37 mike Exp $"
+ * "$Id: texttops.c 6003 2006-10-02 16:26:04Z mike $"
  *
  *   Text to PostScript filter for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 1993-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,12 +15,14 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
+ *   This file is subject to the Apple OS-Developed Software exception.
+ *
  * Contents:
  *
  *   main()          - Main entry for text to PostScript filter.
@@ -79,7 +81,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 void
 WriteEpilogue(void)
 {
-  puts("%%BeginTrailer");
+  puts("%%Trailer");
   printf("%%%%Pages: %d\n", NumPages);
   puts("%%EOF");
 
@@ -103,22 +105,6 @@ WritePage(void)
 
   puts("gsave");
 
-  if (getenv("PPD") == NULL)
-  {
-    switch (Orientation)
-    {
-      case 1 : /* Landscape */
-          printf("%.3f 0.0 translate 90 rotate\n", PageLength);
-          break;
-      case 2 : /* Reverse Portrait */
-          printf("%.3f %.3f translate 180 rotate\n", PageWidth, PageLength);
-          break;
-      case 3 : /* Reverse Landscape */
-          printf("0.0 %.3f translate -90 rotate\n", PageWidth);
-          break;
-    }
-  }
-
   if (PrettyPrint)
     printf("%d H\n", NumPages);
 
@@ -215,17 +201,15 @@ WriteProlog(const char *title,		/* I - Title of job */
 
   curtime = time(NULL);
   curtm   = localtime(&curtime);
-  strftime(curdate, sizeof(curdate), CUPS_STRFTIME_FORMAT, curtm);
+  strftime(curdate, sizeof(curdate), "%c", curtm);
 
   puts("%!PS-Adobe-3.0");
-  printf("%%%%BoundingBox: %.0f %.0f %.0f %.0f\n", PageLeft, PageBottom,
-         PageRight, PageTop);
-  if (Orientation & 1)
-    puts("%%Orientation: Landscape");
+  printf("%%%%BoundingBox: 0 0 %.0f %.0f\n", PageWidth, PageLength);
+  printf("%%cupsRotation: %d\n", (Orientation & 3) * 90);
   puts("%%Creator: texttops/" CUPS_SVERSION);
   printf("%%%%CreationDate: %s\n", curdate);
-  printf("%%%%Title: %s\n", title);
-  printf("%%%%For: %s\n", user);
+  WriteTextComment("Title", title);
+  WriteTextComment("For", user);
   puts("%%Pages: (atend)");
 
  /*
@@ -310,7 +294,7 @@ WriteProlog(const char *title,		/* I - Title of job */
     */
 
     line[strlen(line) - 1] = '\0'; /* Drop \n */
-    for (lineptr = line + 7; isspace(*lineptr); lineptr ++); /* Skip whitespace */
+    for (lineptr = line + 7; isspace(*lineptr & 255); lineptr ++); /* Skip whitespace */
 
     if (strcmp(lineptr, "8bit") == 0)
     {
@@ -345,7 +329,7 @@ WriteProlog(const char *title,		/* I - Title of job */
         start = strtol(lineptr, &lineptr, 16);
 	end   = strtol(lineptr, &lineptr, 16);
 
-	while (isspace(*lineptr))
+	while (isspace(*lineptr & 255))
 	  lineptr ++;
 
         if (!*lineptr)
@@ -353,7 +337,7 @@ WriteProlog(const char *title,		/* I - Title of job */
 
 	valptr = lineptr;
 
-	while (!isspace(*lineptr) && *lineptr)
+	while (!isspace(*lineptr & 255) && *lineptr)
 	  lineptr ++;
 
 	if (!*lineptr)
@@ -384,12 +368,12 @@ WriteProlog(const char *title,		/* I - Title of job */
 	* Got the direction, now get the width...
 	*/
 
-	while (isspace(*lineptr))
+	while (isspace(*lineptr & 255))
 	  lineptr ++;
 
 	valptr = lineptr;
 
-	while (!isspace(*lineptr) && *lineptr)
+	while (!isspace(*lineptr & 255) && *lineptr)
 	  lineptr ++;
 
 	if (!*lineptr)
@@ -422,12 +406,12 @@ WriteProlog(const char *title,		/* I - Title of job */
 
 	for (i = 0; *lineptr && i < 4; i ++)
 	{
-	  while (isspace(*lineptr))
+	  while (isspace(*lineptr & 255))
 	    lineptr ++;
 
 	  valptr = lineptr;
 
-	  while (!isspace(*lineptr) && *lineptr)
+	  while (!isspace(*lineptr & 255) && *lineptr)
 	    lineptr ++;
 
           if (*lineptr)
@@ -512,12 +496,12 @@ WriteProlog(const char *title,		/* I - Title of job */
         start = strtol(lineptr, &lineptr, 16);
 	end   = strtol(lineptr, &lineptr, 16);
 
-	while (isspace(*lineptr))
+	while (isspace(*lineptr & 255))
 	  lineptr ++;
 
 	valptr = lineptr;
 
-	while (!isspace(*lineptr) && *lineptr)
+	while (!isspace(*lineptr & 255) && *lineptr)
 	  lineptr ++;
 
 	if (!*lineptr)
@@ -548,12 +532,12 @@ WriteProlog(const char *title,		/* I - Title of job */
 	* Got the direction, now get the width...
 	*/
 
-	while (isspace(*lineptr))
+	while (isspace(*lineptr & 255))
 	  lineptr ++;
 
 	valptr = lineptr;
 
-	while (!isspace(*lineptr) && *lineptr)
+	while (!isspace(*lineptr & 255) && *lineptr)
 	  lineptr ++;
 
 	if (!*lineptr)
@@ -586,12 +570,12 @@ WriteProlog(const char *title,		/* I - Title of job */
 
 	for (i = 0; *lineptr && i < 4; i ++)
 	{
-	  while (isspace(*lineptr))
+	  while (isspace(*lineptr & 255))
 	    lineptr ++;
 
 	  valptr = lineptr;
 
-	  while (!isspace(*lineptr) && *lineptr)
+	  while (!isspace(*lineptr & 255) && *lineptr)
 	    lineptr ++;
 
           if (*lineptr)
@@ -715,7 +699,9 @@ WriteProlog(const char *title,		/* I - Title of job */
     else
       j = 0;
 
-    if (ppd != NULL && j >= ppd->num_fonts)
+    if ((ppd != NULL && j >= ppd->num_fonts) ||
+        strncmp(fonts[i], "Courier", 7) == 0 ||
+	strcmp(fonts[i], "Symbol") == 0)
     {
      /*
       * Need to embed this font...
@@ -744,7 +730,9 @@ WriteProlog(const char *title,		/* I - Title of job */
     else
       j = 0;
 
-    if (ppd != NULL && j >= ppd->num_fonts)
+    if ((ppd != NULL && j >= ppd->num_fonts) ||
+        strncmp(fonts[i], "Courier", 7) == 0 ||
+	strcmp(fonts[i], "Symbol") == 0)
     {
      /*
       * Need to embed this font...
@@ -911,17 +899,24 @@ WriteProlog(const char *title,		/* I - Title of job */
 
     puts("/n {");
     puts("\t20 string cvs % convert page number to string");
-    puts("\tdup length % get length");
-    puts("\tdup 2 mul string /P exch def % P = string twice as long");
-    puts("\t0 1 2 index 1 sub { % loop through each character in the page number");
-    puts("\t\tdup 3 index exch get % get character N from the page number");
-    puts("\t\texch 2 mul dup % compute offset in P");
-    puts("\t\tP exch 0 put % font 0");
-    puts("\t\t1 add P exch 2 index put % character");
-    puts("\t\tpop % discard character");
-    puts("\t} for % do for loop");
-    puts("\tpop pop % discard string and length");
-    puts("\tP % put string on stack");
+    if (NumFonts > 1)
+    {
+     /*
+      * Convert a number to double-byte chars...
+      */
+
+      puts("\tdup length % get length");
+      puts("\tdup 2 mul string /P exch def % P = string twice as long");
+      puts("\t0 1 2 index 1 sub { % loop through each character in the page number");
+      puts("\t\tdup 3 index exch get % get character N from the page number");
+      puts("\t\texch 2 mul dup % compute offset in P");
+      puts("\t\tP exch 0 put % font 0");
+      puts("\t\t1 add P exch 2 index put % character");
+      puts("\t\tpop % discard character");
+      puts("\t} for % do for loop");
+      puts("\tpop pop % discard string and length");
+      puts("\tP % put string on stack");
+    }
     puts("} bind def");
 
     printf("/T");
@@ -933,7 +928,7 @@ WriteProlog(const char *title,		/* I - Title of job */
     puts("def");
 
     puts("/H {");
-    puts("gsave");
+    puts("\tgsave");
     puts("\t0.9 setgray");
 
     if (Duplex)
@@ -1084,7 +1079,7 @@ write_line(int     row,		/* I - Row number (0 to N) */
 	while (col < SizeColumns && line->ch != 0 && attr == line->attr)
 	{
           if (Directions[Chars[line->ch] / 256] > 0 &&
-	      !ispunct(line->ch) && !isspace(line->ch))
+	      !ispunct(line->ch & 255) && !isspace(line->ch & 255))
 	    break;
 
 	  col ++;
@@ -1092,7 +1087,7 @@ write_line(int     row,		/* I - Row number (0 to N) */
 	}
 
         for (i = 1; start < line; i ++, start ++)
-	  if (!isspace(start->ch))
+	  if (!isspace(start->ch & 255))
 	    write_string(col - i, row, 1, start);
       }
     }
@@ -1312,5 +1307,5 @@ write_text(const char *s)	/* I - String to write */
 
 
 /*
- * End of "$Id: texttops.c,v 1.40 2002/01/30 16:32:37 mike Exp $".
+ * End of "$Id: texttops.c 6003 2006-10-02 16:26:04Z mike $".
  */
