@@ -1,5 +1,5 @@
 /*
- * "$Id: cups-lpd.c 5455 2006-04-24 13:49:59Z mike $"
+ * "$Id: cups-lpd.c 6022 2006-10-10 19:47:03Z mike $"
  *
  *   Line Printer Daemon interface for the Common UNIX Printing System (CUPS).
  *
@@ -150,8 +150,6 @@ main(int  argc,				/* I - Number of command-line arguments */
   num_defaults = 0;
   defaults     = NULL;
   hostlookups  = 1;
-  num_defaults = cupsAddOption("job-originating-host-name", hostname,
-                               num_defaults, &defaults);
 
   for (i = 1; i < argc; i ++)
     if (argv[i][0] == '-')
@@ -217,6 +215,9 @@ main(int  argc,				/* I - Number of command-line arguments */
            hostip);
   }
 
+  num_defaults = cupsAddOption("job-originating-host-name", hostname,
+                               num_defaults, &defaults);
+
  /*
   * RFC1179 specifies that only 1 daemon command can be received for
   * every connection.
@@ -471,7 +472,7 @@ get_printer(http_t        *http,	/* I - HTTP connection */
 
     request = ippNewRequest(CUPS_GET_PRINTERS);
 
-    ippAddStrings(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+    ippAddStrings(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
                   "requested-attributes",
 		  (int)(sizeof(requested) / sizeof(requested[0])),
                   NULL, requested);
@@ -597,7 +598,7 @@ get_printer(http_t        *http,	/* I - HTTP connection */
     ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri",
         	 NULL, uri);
 
-    ippAddStrings(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+    ippAddStrings(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
                   "requested-attributes",
 		  (int)(sizeof(requested) / sizeof(requested[0])),
                   NULL, requested);
@@ -648,7 +649,7 @@ get_printer(http_t        *http,	/* I - HTTP connection */
     if (state)
     {
       if ((attr = ippFindAttribute(response, "printer-state",
-                        	   IPP_TAG_INTEGER)) == NULL)
+                        	   IPP_TAG_ENUM)) == NULL)
 	syslog(LOG_ERR, "No printer-state attribute found in "
                 	"response from server!");
       else
@@ -1387,7 +1388,7 @@ remove_jobs(const char *dest,		/* I - Destination */
       return (1);
     }
     else
-      syslog(LOG_INFO, "Job ID %d cancelled", id);
+      syslog(LOG_INFO, "Job ID %d canceled", id);
   }
 
   httpClose(http);
@@ -1716,5 +1717,5 @@ smart_gets(char *s,			/* I - Pointer to line buffer */
 
 
 /*
- * End of "$Id: cups-lpd.c 5455 2006-04-24 13:49:59Z mike $".
+ * End of "$Id: cups-lpd.c 6022 2006-10-10 19:47:03Z mike $".
  */
