@@ -1,5 +1,5 @@
 /*
- * "$Id: subscriptions.c 5716 2006-07-11 17:56:57Z mike $"
+ * "$Id: subscriptions.c 5991 2006-09-29 02:26:29Z mike $"
  *
  *   Subscription routines for the Common UNIX Printing System (CUPS) scheduler.
  *
@@ -50,6 +50,11 @@
 #include "cupsd.h"
 #ifdef HAVE_DBUS
 #  include <dbus/dbus.h>
+#  ifdef HAVE_DBUS_MESSAGE_ITER_INIT_APPEND
+#    define dbus_message_append_iter_init dbus_message_iter_init_append
+#    define dbus_message_iter_append_string(i,v) dbus_message_iter_append_basic(i, DBUS_TYPE_STRING, v)
+#    define dbus_message_iter_append_uint32(i,v) dbus_message_iter_append_basic(i, DBUS_TYPE_UINT32, v)
+#  endif /* HAVE_DBUS_MESSAGE_ITER_INIT_APPEND */
 #endif /* HAVE_DBUS */
 
 
@@ -267,7 +272,7 @@ cupsdAddEvent(
 			   "job-stopped");
               break;
 
-	  case IPP_JOB_CANCELLED :
+	  case IPP_JOB_CANCELED :
               ippAddString(temp->attrs, IPP_TAG_EVENT_NOTIFICATION,
 		           IPP_TAG_KEYWORD, "job-state-reasons", NULL,
 			   "job-canceled-by-user");
@@ -1245,13 +1250,12 @@ cupsdStopAllNotifiers(void)
 void
 cupsdUpdateNotifierStatus(void)
 {
-  char		*ptr,			/* Pointer to end of line in buffer */
-		message[1024];		/* Pointer to message text */
+  char		message[1024];		/* Pointer to message text */
   int		loglevel;		/* Log level for message */
 
 
-  while ((ptr = cupsdStatBufUpdate(NotifierStatusBuffer, &loglevel,
-                                   message, sizeof(message))) != NULL)
+  while (cupsdStatBufUpdate(NotifierStatusBuffer, &loglevel,
+                            message, sizeof(message)))
     if (!strchr(NotifierStatusBuffer->buffer, '\n'))
       break;
 }
@@ -1356,13 +1360,13 @@ cupsd_send_dbus(cupsd_eventmask_t event,/* I - Event to send */
   message = dbus_message_new_signal("/com/redhat/PrinterSpooler",
 				    "com.redhat.PrinterSpooler", what);
 
-  dbus_message_iter_init_append(message, &iter);
+  dbus_message_append_iter_init(message, &iter);
   if (dest)
-    dbus_message_iter_append_basic(&iter, DBUS_TYPE_STRING, &(dest->name));
+    dbus_message_iter_append_string(&iter, &(dest->name));
   if (job)
   {
-    dbus_message_iter_append_basic(&iter, DBUS_TYPE_UINT32, &(job->id));
-    dbus_message_iter_append_basic(&iter, DBUS_TYPE_STRING, &(job->username));
+    dbus_message_iter_append_uint32(&iter, &(job->id));
+    dbus_message_iter_append_string(&iter, &(job->username));
   }
 
   dbus_connection_send(con, message, NULL);
@@ -1506,7 +1510,6 @@ cupsd_start_notifier(
 {
   int	pid;				/* Notifier process ID */
   int	fds[2];				/* Pipe file descriptors */
-  int	envc;				/* Number of environment variables */
   char	*argv[4],			/* Command-line arguments */
 	*envp[MAX_ENV],			/* Environment variables */
 	user_data[128],			/* Base-64 encoded user data */
@@ -1546,7 +1549,7 @@ cupsd_start_notifier(
   * Setup the environment...
   */
 
-  envc = cupsdLoadEnv(envp, (int)(sizeof(envp) / sizeof(envp[0])));
+  cupsdLoadEnv(envp, (int)(sizeof(envp) / sizeof(envp[0])));
 
  /*
   * Create pipes as needed...
@@ -1624,5 +1627,5 @@ cupsd_start_notifier(
 
 
 /*
- * End of "$Id: subscriptions.c 5716 2006-07-11 17:56:57Z mike $".
+ * End of "$Id: subscriptions.c 5991 2006-09-29 02:26:29Z mike $".
  */
