@@ -1,5 +1,5 @@
 /*
- * "$Id: cupstestppd.c 5549 2006-05-19 19:39:28Z mike $"
+ * "$Id: cupstestppd.c 5926 2006-09-05 20:45:47Z mike $"
  *
  *   PPD test program for the Common UNIX Printing System (CUPS).
  *
@@ -113,6 +113,8 @@ main(int  argc,			/* I - Number of command-line arguments */
                                 "JCL", "PAGE", "PROLOG" };
 
 
+  _cupsSetLocale(argv);
+
  /*
   * Display PPD files for each file listed on the command-line...
   */
@@ -306,9 +308,9 @@ main(int  argc,			/* I - Number of command-line arguments */
       * Look for default keywords with no matching option...
       */
 
-      for (i = 0; i < ppd->num_attrs; i ++)
+      for (j = 0; j < ppd->num_attrs; j ++)
       {
-	attr = ppd->attrs[i];
+	attr = ppd->attrs[j];
 
         if (!strcmp(attr->name, "DefaultColorSpace") ||
 	    !strcmp(attr->name, "DefaultFont") ||
@@ -327,11 +329,11 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    * Check that the default option value matches a choice...
 	    */
 
-	    for (j = 0; j < option->num_choices; j ++)
-	      if (!strcmp(option->choices[j].choice, attr->value))
+	    for (k = 0; k < option->num_choices; k ++)
+	      if (!strcmp(option->choices[k].choice, attr->value))
 	        break;
 
-            if (j >= option->num_choices)
+            if (k >= option->num_choices)
 	    {
 	      if (verbose >= 0)
 	      {
@@ -1010,7 +1012,10 @@ main(int  argc,			/* I - Number of command-line arguments */
         char	*languages,		/* Copy of attribute value */
 		*langstart,		/* Start of current language */
 		*langptr,		/* Pointer into languages */
-		keyword[PPD_MAX_NAME];	/* Localization keyword */
+		keyword[PPD_MAX_NAME],	/* Localization keyword */
+		ckeyword[PPD_MAX_NAME];	/* Custom option keyword */
+	ppd_coption_t	*coption;	/* Custom option */
+	ppd_cparam_t	*cparam;	/* Custom parameter */
 
 
         languages = strdup(attr->value);
@@ -1104,7 +1109,61 @@ main(int  argc,			/* I - Number of command-line arguments */
 	             option->keyword);
             for (j = 0; j < option->num_choices; j ++)
 	    {
-	      if (!ppdFindAttr(ppd, keyword, option->choices[j].choice))
+	      if (!strcasecmp(option->choices[j].choice, "Custom") &&
+	          (coption = ppdFindCustomOption(ppd,
+		                                 option->keyword)) != NULL)
+	      {
+		snprintf(ckeyword, sizeof(ckeyword), "%s.Custom%s",
+		         langstart, option->keyword);
+
+		if (!ppdFindAttr(ppd, ckeyword, "True"))
+		{
+		  if (verbose >= 0)
+		  {
+		    if (!errors && !verbose)
+		      _cupsLangPuts(stdout, _(" FAIL\n"));
+
+		    _cupsLangPrintf(stdout,
+	                	    _("      **FAIL**  Missing \"%s\" "
+				      "translation string for option %s, "
+				      "choice %s!\n"),
+				    langstart, ckeyword + 1 + strlen(langstart),
+				    "True");
+        	  }
+
+		  errors ++;
+		}
+
+                if (strcasecmp(option->keyword, "PageSize"))
+		{
+		  for (cparam = (ppd_cparam_t *)cupsArrayFirst(coption->params);
+		       cparam;
+		       cparam = (ppd_cparam_t *)cupsArrayNext(coption->params))
+		  {
+		    snprintf(ckeyword, sizeof(ckeyword), "%s.ParamCustom%s",
+		             langstart, option->keyword);
+		    if (!ppdFindAttr(ppd, ckeyword, cparam->name))
+		    {
+		      if (verbose >= 0)
+		      {
+			if (!errors && !verbose)
+			  _cupsLangPuts(stdout, _(" FAIL\n"));
+
+			_cupsLangPrintf(stdout,
+	                		_("      **FAIL**  Missing \"%s\" "
+					  "translation string for option %s, "
+					  "choice %s!\n"),
+					langstart,
+					ckeyword + 1 + strlen(langstart),
+				        cparam->name);
+        	      }
+
+		      errors ++;
+		    }
+                  }
+                }
+	      }
+	      else if (!ppdFindAttr(ppd, keyword, option->choices[j].choice))
 	      {
 		if (verbose >= 0)
 		{
@@ -1184,9 +1243,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	  if (!errors && !verbose)
 	    _cupsLangPuts(stdout, _(" FAIL\n"));
 
-	  _cupsLangPuts(stdout,
-	                _("      **FAIL**  1284DeviceId must be 1284DeviceID!\n"
-			  "                REF: Page 72, section 5.5\n"));
+	  _cupsLangPrintf(stdout,
+	                  _("      **FAIL**  %s must be 1284DeviceID!\n"
+			    "                REF: Page 72, section 5.5\n"),
+			  attr->name);
         }
 
 	errors ++;
@@ -1205,9 +1265,9 @@ main(int  argc,			/* I - Number of command-line arguments */
 	* Look for default keywords with no corresponding option...
 	*/
 
-	for (i = 0; i < ppd->num_attrs; i ++)
+	for (j = 0; j < ppd->num_attrs; j ++)
 	{
-	  attr = ppd->attrs[i];
+	  attr = ppd->attrs[j];
 
           if (!strcmp(attr->name, "DefaultColorSpace") ||
 	      !strcmp(attr->name, "DefaultColorSep") ||
@@ -1730,5 +1790,5 @@ usage(void)
 
 
 /*
- * End of "$Id: cupstestppd.c 5549 2006-05-19 19:39:28Z mike $".
+ * End of "$Id: cupstestppd.c 5926 2006-09-05 20:45:47Z mike $".
  */
